#import "TCAppKitExtensions.h"

#define TCToolBarCurrentVersion			1

#define TCToolBarDefaultIntercellSpacing	4.0	// spacing between cells
#define TCToolBarDefaultOriginLeft		0.0	// left origin
#define TCToolBarDefaultBorderType		NSNoBorder

@implementation TCToolBar

+ (void)initialize
{
    if (self == [TCToolBar class]) {
        [self setVersion:TCToolBarCurrentVersion];
    }
}

- (void)_defaultInit
{
    cellDictionary = [[NSMutableDictionary alloc] init];
    cellSize = NSMakeSize([self frame].size.height, [self frame].size.height);
    intercellSpacing = TCToolBarDefaultIntercellSpacing;
    nextCellOriginLeft = TCToolBarDefaultOriginLeft;
    gradientType = NSGradientConcaveWeak;
    borderType = TCToolBarDefaultBorderType;
}

- (id)initWithFrame:(NSRect)frame
{
    if (self = [super initWithFrame:frame])
        [self _defaultInit];
    
    return self;
}

- (void)dealloc
{
    [cellDictionary release];

    [super dealloc];
}

- (void)drawRect:(NSRect)clipRect
{
    switch(borderType) {
        case     NSNoBorder:
            break;
            
        case NSLineBorder:
            PSsetgray(NSBlack);
            NSFrameRect(_frame);
            break;
            
        case NSBezelBorder:
            NSDrawGrayBezel(_frame, clipRect);
            break;
            
        case NSGrooveBorder:
            NSDrawGroove(_frame, clipRect);
            break;
            
        default:
            TCLog(self, _cmd, @"Warning: unknown border type %d", borderType);
            break;
    }

    [super drawRect:clipRect];
}

- (NSBorderType)borderType
{
    return borderType;
}

- (void)setBorderType:(NSBorderType)type
{
    borderType = type;
    [self setNeedsDisplay:YES];
}

- (NSGradientType)gradientType
{
    return gradientType;
}

- (void)setGradientType:(NSGradientType)type
{
    gradientType = type;
    [self setNeedsDisplay:YES];
}

- (NSSize)cellSize
{
    return cellSize;
}

- (void)setCellSize:(NSSize)size
{
    cellSize = size;
    [self setNeedsDisplay:YES];
}

- (float)intercellSpacing
{
    return intercellSpacing;
}

- (void)setIntercellSpacing:(float)width
{
    intercellSpacing = width;
    [self setNeedsDisplay:YES];
}

- (NSButton *)cellNamed:(NSString *)name
{
    return [cellDictionary objectForKey:name];
}

- (NSArray *)cells
{
    return [cellDictionary allValues];
}

- (void)addCellName:(NSString *)name icon:(NSImage *)iconImage toolTip:(NSString *)toolTip preceedingSeparator:(BOOL)separatorFlag target:(id)target action:(SEL)action
{
    NSRect newFrame;
    NSButton *newButton;

    if (separatorFlag)
        nextCellOriginLeft += intercellSpacing;

    newFrame = NSMakeRect(nextCellOriginLeft, 0, cellSize.width, cellSize.height);
    newButton = [[[NSButton alloc] initWithFrame:newFrame] autorelease];

    [newButton setImage:iconImage];
    [newButton setToolTip:toolTip];
    [newButton setTarget:target];
    [newButton setAction:action];
    [[newButton cell] setGradientType:gradientType];

    [self addSubview:newButton];

    nextCellOriginLeft += (cellSize.width + 1);

    [self setNeedsDisplay:YES];
}

- (id)initWithCoder:(NSCoder *)decoder
{
    if (self = [super initWithCoder:decoder]) {
        switch ([decoder versionForClassName:[[self class] description]]) {
            case TCToolBarCurrentVersion:
                cellDictionary = [[decoder decodeObject] retain];
                [decoder decodeValueOfObjCType:@encode(NSSize) at:&cellSize];
                [decoder decodeValueOfObjCType:@encode(float) at:&intercellSpacing];
                [decoder decodeValueOfObjCType:@encode(float) at:&nextCellOriginLeft];
                [decoder decodeValueOfObjCType:@encode(NSGradientType) at:&gradientType];
                [decoder decodeValueOfObjCType:@encode(NSBorderType) at:&borderType];
                break;

            default:				// no version, so we use defaults...
                TCLog(self, _cmd, @"Warning: unknown class version %d",
                      [decoder versionForClassName:[[self class] description]]);
                [self _defaultInit];

                break;
        }
    }

    return self;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    [super encodeWithCoder:coder];

    [coder encodeObject:cellDictionary];
    [coder encodeValueOfObjCType:@encode(NSSize) at:&cellSize];
    [coder encodeValueOfObjCType:@encode(float) at:&intercellSpacing];
    [coder encodeValueOfObjCType:@encode(float) at:&nextCellOriginLeft];
    [coder encodeValueOfObjCType:@encode(NSGradientType) at:&gradientType];
    [coder encodeValueOfObjCType:@encode(NSBorderType) at:&borderType];
}

@end
