/*
 * TCNSArrayExtensions.m
 * Category of extensions to the NSArray Foundation class.
 *
 * Copyright (c) 1997 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1997.
 */

/* NeXT headers */
#import <Foundation/Foundation.h>

/* Local headers */
#import "TCFoundationExtensions.h"

@implementation NSArray (NSArrayExtensions)

+ (NSArray *)arrayWithCStringArray:(char **)cStringArray
    /*" Returns an autoreleased array of NSStrings created from the
    NULL-terminated array of C-Strings cStringArray. "*/
{
    return [[[self alloc] initWithCStringArray:cStringArray] autorelease];
}

- initWithCStringArray:(char **)cStringArray
    /*" Returns an array of NSStrings created from the NULL-terminated
    array of C-Strings cStringArray. "*/
{
    NSMutableArray	*array = [NSMutableArray new];
    char *foo;

    while (foo = *cStringArray++)
        [array addObject:[NSString stringWithCString:foo]];

    [self release];
    return [array copy];
}

- (const char **)cStringArray
{
    char **cStringArray = [[NSMutableData dataWithLength:
        ([self count]+1)*sizeof(char *)] mutableBytes];
    unsigned i;

    for (i = 0; i < [self count]; ++i) {
        cStringArray[i] = (char *)[[(NSObject *)[self objectAtIndex:i]
            description] cString];
        }

    return cStringArray;
}

- (void)makeObjectsPerformSelector:(SEL)aSelector
                        withObject:anObject
                        withObject:anotherObject
/*" An extension of #makeObjectsPerform:withObject:. "*/
{
    NSEnumerator *e = [self objectEnumerator];
    id obj;

    while (obj = [e nextObject])
        if ([obj respondsToSelector:aSelector])
            [obj performSelector:aSelector
                      withObject:anObject
                      withObject:anotherObject];
}

@end

@implementation NSMutableArray (TCNSMutableArrayExtensions)

- (void)removeObjectsEqualTo:anObject
{
    while ([self containsObject:anObject])
        [self removeObject:anObject];
}

@end

