#import "TCFoundationExtensions.h"

@implementation NSBundle(MultipleLoadingConveniences)

+ (NSArray *)bundlesWithExtension:(NSString *)suffix paths:(NSArray *)paths
{
    NSEnumerator *objectEnumerator;
    NSString *path;
    NSMutableArray *bundles = [NSMutableArray array];

    // once built, we allocate and initialize the apprentice instances
    objectEnumerator = [paths objectEnumerator];
    while (path = [objectEnumerator nextObject]) {
        NSDirectoryEnumerator *directoryEnumerator;
        NSString *bundleName;

        directoryEnumerator = [[NSFileManager defaultManager] enumeratorAtPath:path];

        while (bundleName = [directoryEnumerator nextObject]) {
            if ([[bundleName pathExtension] isEqualToString:suffix]) {
                NSString *bundlePath = [path stringByAppendingPathComponent:bundleName];
                NSBundle *newBundle = [NSBundle bundleWithPath:bundlePath];

                if (newBundle != nil)
                    [bundles addObject:newBundle];
            }
        }
    }

    return bundles;
}

+ (NSArray *)bundlesWithExtension:(NSString *)suffix pathComponent:(NSString *)pathSuffix
{
    NSMutableArray *searchPaths = [NSMutableArray array];
    NSMutableArray *bundlesAndFrameworks = [NSMutableArray array];
    NSEnumerator *objectEnumerator;
    NSString *path;
    NSBundle *bundle;

    [bundlesAndFrameworks addObjectsFromArray:[NSBundle allBundles]];
    [bundlesAndFrameworks addObjectsFromArray:[NSBundle allFrameworks]];

    // first we build an array containing all the paths we'd like to search.
    objectEnumerator = [bundlesAndFrameworks objectEnumerator];
    while (bundle = [objectEnumerator nextObject])
        [searchPaths addObject:[bundle resourcePath]];

#if defined(__APPLE__)
    objectEnumerator = [NSSearchPathForDirectoriesInDomains(NSAllLibrariesDirectory, NSAllDomainsMask, YES) objectEnumerator];
#else
    objectEnumerator = [NSStandardLibraryPaths() objectEnumerator];
#endif
    
    while (path = [objectEnumerator nextObject])
        [searchPaths addObject:[path stringByAppendingPathComponent:pathSuffix]];

    return [self bundlesWithExtension:suffix paths:searchPaths];
}

@end
