/*
 * TCNSDataExtensions.m
 * Category of extensions to the NSData Foundation class.
 *
 * Copyright (c) 1997 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1997.
 */

/* NeXT headers */
#import <Foundation/Foundation.h>

/* Local headers */
#import "NSString+TCExtensions.h"
#import "NSData+TCExtensions.h"

@implementation NSData (TCNSDataExtensions)

+ (NSData *)dataWithOutputOfTask:(NSTask *)shellTask
{
    return [[[self alloc] initWithOutputOfTask:shellTask] autorelease];
}

+ (NSData *)dataWithOutputOfTask:(NSTask *)shellTask inputData:(NSData *)stdinData
{
    return [[[self alloc] initWithOutputOfTask:shellTask inputData:stdinData] autorelease];
}

+ (NSData *)dataWithOutputOfTask:(NSTask *)shellTask inputString:(NSString *)stdinString
{
    return [[[self alloc] initWithOutputOfTask:shellTask inputString:stdinString] autorelease];
}

- (id)initWithOutputOfTask:(NSTask *)shellTask
{
    NSPipe *pipe = [NSPipe pipe];
    NSFileHandle *readHandle = [pipe fileHandleForReading];
    NSData *dataPacket;
    NSMutableData *totalData = [NSMutableData new];

    [shellTask retain];
    [shellTask setStandardOutput:pipe];
    [shellTask launch];

    while ((dataPacket = [readHandle availableData]) && [dataPacket length]) {
        [totalData appendData:dataPacket];
    }

    [self autorelease];
    [shellTask release];
    return [totalData copy];
}

- (id)initWithOutputOfTask:(NSTask *)shellTask inputData:(NSData *)stdinData
{
    NSPipe *taskOutPipe = [NSPipe pipe],
        *taskInPipe = [NSPipe pipe];
    NSFileHandle *readHandle = [taskOutPipe fileHandleForReading],
        *writeHandle = [taskInPipe fileHandleForWriting];
    NSData *dataPacket;
    NSMutableData *totalData = [NSMutableData new];

    [shellTask retain];
    [shellTask setStandardOutput:taskOutPipe];
    [shellTask setStandardInput:taskInPipe];
    [shellTask launch];

    [writeHandle writeData:stdinData];
    [writeHandle closeFile];
    
    while ((dataPacket = [readHandle availableData]) && [dataPacket length]) {
        [totalData appendData:dataPacket];
    }

    [self autorelease];
    [shellTask release];
    return [totalData copy];
}

- (id)initWithOutputOfTask:(NSTask *)shellTask inputString:(NSString *)stdinString
{
    return [self initWithOutputOfTask:shellTask
                            inputData:[stdinString dataUsingEncoding:NSASCIIStringEncoding]];
}

@end