/* NSString+TCExtensions.m created by dwy on Sat 03-Jan-1998 */

#import <Foundation/Foundation.h>
#import "NSString+TCExtensions.h"

#import <ctype.h>

@implementation NSString(TCExtensions)

+ (NSString *)stringWithData:(NSData *)data encoding:(NSStringEncoding)encoding
{
    return [[[NSString alloc] initWithData:data encoding:encoding] autorelease];
}

+ (NSString *)stringWithContentsOfFile:(NSString *)fileName
{
    return [[[self alloc] initWithContentsOfFile:fileName] autorelease];
}

+ (NSString *)stringWithTemporaryFileName
{
    return [[[self alloc] initWithTemporaryFileName] autorelease];
}

- (NSString *)stringByExpandingPathInHomeDirectory
{
    return [NSHomeDirectory() stringByAppendingPathComponent:self];
}

- (NSString *)stringByExpandingPathInLibraryDirectory
{
    return [[@"Library" stringByExpandingPathInHomeDirectory]
        stringByAppendingPathComponent:self];
}

- (NSString *)stringByPrependingString:(NSString *)aString
    /*" Returns the result of concatenating this string onto aString. "*/
{
    NSString *newString;

    newString = [aString copy];
    return [newString stringByAppendingString:self];
}

- (NSString *)stringByRemovingPrefix:(NSString *)aPrefix
    /*" Returns the result of stripping aPrefix from the beginning
    of this string. "*/
{
    NSRange r;

    r = [self rangeOfString:aPrefix];
    return [self substringFromIndex:r.length];
}

- (NSString* )stringByRemovingSuffix:(NSString *)aSuffix
    /*" Returns the result of stripping aSuffix from the end
    of this string. "*/
{
    NSRange r;

    r = [self rangeOfString:aSuffix options:NSBackwardsSearch];
    return [self substringToIndex:r.location];
}

+ (NSString *)stringWithSelector:(SEL)aSel
    /*" Returns a string with aSel's name. "*/
{
    return NSStringFromSelector(aSel);
}

- (SEL)selectorValue
    /*" Returns this string's selector value. "*/
{
    return (NSSelectorFromString(self));
}

int fnmatch (const char *pattern, const char *string, int flags);

- (BOOL)matchesWildcard:(NSString *)wildcard
{
    return (fnmatch([self cString], [wildcard cString], 0) == 0);
}

- (NSString *)stringByPaddingToByteBoundary:(unsigned)boundary
                                      padCharacter:(NSString *)padChar
{
    unsigned i = [self length];
    NSMutableString *str = [NSMutableString new];

    while ((i % boundary) != 0)
        i++;

    while (i > [self length]) {
        [str appendString:padChar];
        --i;
        }

    return [NSString stringWithFormat:@"%@%@", self, str];
}

- (NSArray *)componentsBrokenIntoPartsOfLength:(unsigned)aLength
{
    NSMutableData *someData = [[self dataUsingEncoding:NSASCIIStringEncoding]
        mutableCopy];
    char *string = [someData mutableBytes];
    unsigned i, partCounter = 0;
    NSMutableArray *returnArray = [NSMutableArray new];

    for (i = 0; i < [someData length]; ++i) {
        partCounter++;
        if (partCounter == aLength) {
            [returnArray addObject:[NSString stringWithCString:string length:aLength]];
            string += aLength;
            partCounter = 0;
        }
    }

    if (partCounter != 0)
        [returnArray addObject:[NSString stringWithCString:string length:partCounter]];

    return [returnArray copy];
}

- (id)initWithTemporaryFileName
{
    return [self initWithCString:tmpnam(NULL)];
}

- (NSString *)stringByRemovingTrailingNulls
{
    return [[self componentsSeparatedByString:@"\000"] objectAtIndex:0];
}

+ (NSString *)stringWithHexBytesInData:(NSData *)data;
{
    static const char hexSet [] = "0123456789abcdef";
    const char *bytes = [data bytes];
    NSMutableData *stringData = [NSMutableData dataWithLength:[data length]*2];
    unsigned char *stringBytes = [stringData mutableBytes];
    unsigned i;

    for (i = 0; i < [data length]; ++i) {
        *stringBytes++ = hexSet[(bytes[i] & 0xf0) / 0x10];
        *stringBytes++ = hexSet[bytes[i] & 0x0f];
    }

    return [NSString stringWithData:stringData encoding:NSASCIIStringEncoding];
}

unsigned char hex2byte (char c)
{
    if (isdigit (c))
        return (unsigned char) (c - '0');
    else if (islower (c))
        return (unsigned char) (10 + c - 'a');
    else
        return (unsigned char) (10 + c - 'A');
}

- (NSData *)dataUsingHexEncoding
{
    NSMutableData *data = [NSMutableData dataWithLength:[self length]/2];
    unsigned char *bytes = [data mutableBytes];
    const char *cString = [self cString];

    while (*cString != 0) 
        *bytes++ = ((hex2byte(*cString++)*0x10) + hex2byte(*cString++));

    return [[data copy] autorelease];
}

@end

@implementation NSMutableString (TCNSMutableStringExtensions)

- (void)appendCString:(char *)cString
    /*" Concatenates the NULL-terminated string cString to this string. "*/
{
    [self appendString:[NSString stringWithCString:cString]];
}

- (void)appendWithFormat:(NSString *)format, ...
{
    va_list argList;
    NSString   *string;

    va_start(argList, format);
    string = [[NSString alloc] initWithFormat:format arguments:argList];
    [self appendString:[string autorelease]];
    va_end(argList);
}

@end
