/*
 * TCAddressResolver.m
 * Implementation for the asynchronous name resolver.
 *
 * Copyright (c) 1997 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1997.
 */

/* NeXT headers */
#import <Foundation/Foundation.h>

/* Project headers */
#import "TCNetworking.h"

DEFINE_NSSTRING(TCAddressResolverCompletionNotification);
DEFINE_NSSTRING(TCAddressResolverUserInfoExceptionNameItemKey);
DEFINE_NSSTRING(TCAddressResolverUserInfoExceptionReasonItemKey);

@implementation TCAddressResolver

static NSPort *_dataPort = nil;

+ (void)detachNewResolverHostName:(NSString *)hostName object:(id)anObject selector:(SEL)aSelector;
{
    NSArray *arguments;

    // create our dataPort, if it's not yet created
    if (_dataPort == nil) {
        _dataPort = [[NSPort alloc] init];
        [_dataPort setDelegate:self];

        [[NSRunLoop currentRunLoop] addPort:_dataPort forMode:NSDefaultRunLoopMode];
    }

    arguments = [NSArray arrayWithObjects:hostName,
        [TCOperation operationWithObject:nil target:anObject selector:aSelector],
        nil];

    [NSThread detachNewThreadSelector:@selector(threadSelector:)
                             toTarget:self
                           withObject:[arguments retain]];
}

+ (void)threadSelector:(NSArray *)arguments
{
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    NSString *string = [arguments objectAtIndex:0];
    TCOperation *operation = [arguments objectAtIndex:1];
    TCOperation *secondOperation;
    NSDictionary *userInfo;

    NS_DURING
        secondOperation = [TCOperation operationWithObject:[NSHost hostWithName:string]
                                                    target:[operation target]
                                                  selector:[operation selector]];
    NS_HANDLER
        userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
            [localException name], TCAddressResolverUserInfoExceptionNameItemKey,
            [localException reason], TCAddressResolverUserInfoExceptionReasonItemKey,
            nil];

        secondOperation = [TCOperation operationWithObject:nil
                                                    target:[operation target]
                                                  selector:[operation selector]
                                                  userInfo:userInfo];
    NS_ENDHANDLER;

    if (![_dataPort sendObject:secondOperation])
        NSLog(@"+[TCAddressResolver threadSelector]: couldn't send mach message for %@",
              [secondOperation description]);
        
    [pool release];
    [NSThread exit];
}

+ (void)handlePortMessage:(NSPortMessage *)message
{
    NSData *serializedObjectData = [[message components] objectAtIndex:0];
    TCOperation *operation = [NSUnarchiver unarchiveObjectWithData:serializedObjectData];

    // Executed the decoded operation
    [[operation target] performSelector:[operation selector] withObject:[operation object]];

    // Post a global notification
    [[NSNotificationCenter defaultCenter]
        postNotificationName:TCAddressResolverCompletionNotification
                      object:[operation object]
                    userInfo:[operation userInfo]];
}

@end
