/*
 * TCOperation.m
 * Private class used to model an asynchronous name resolution operation.
 *
 * Copyright (c) 1997 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1997.
 */

/* NeXT headers */
#import <Foundation/Foundation.h>

/* Project headers */
#import "TCNetworking.h"

@implementation TCOperation

+ (TCOperation *)operationWithObject:(id)object target:(id)target selector:(SEL)selector
{
    return [[[self alloc] initWithObject:object target:target selector:selector userInfo:nil] autorelease];
}

+ (TCOperation *)operationWithObject:(id)object target:(id)target selector:(SEL)selector userInfo:(NSDictionary *)userInfo
{
    return [[[self alloc] initWithObject:object target:target selector:selector userInfo:userInfo] autorelease];
}

- (id)initWithObject:(id)object target:(id)target selector:(SEL)selector userInfo:(NSDictionary *)userInfo
{
    if (self = [super init]) {
        _object = [object retain];
        _target = [target retain];
        _selector = selector;
        _userInfo = [userInfo retain];
    }

    return self;
}

- (void)dealloc
{
    [_object release];
    [_target release];
    [_userInfo release];
    
    [super dealloc];
}

- (NSString *)description
{
    return [NSString stringWithFormat:@"TCOperation object=(%@) target=0x%lx selector=%@",
        [_object description], _target, NSStringFromSelector(_selector)];
}

- (id)object
{
    return _object;
}

- (id)target
{
    return _target;
}

- (SEL)selector
{
    return _selector;
}

- (NSDictionary *)userInfo
{
    return [[_userInfo copy] autorelease];
}

- (id)initWithCoder:(NSCoder *)decoder
{
    if (self = [super init]) {
        _object = [[decoder decodeObject] retain];
        _userInfo = [[decoder decodeObject] retain];
        [decoder decodeValueOfObjCType:@encode(unsigned long) at:&_target];
        [decoder decodeValueOfObjCType:@encode(SEL) at:&_selector];
    }

    return self;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    [coder encodeObject:_object];
    [coder encodeObject:_userInfo];
    [coder encodeValueOfObjCType:@encode(unsigned long) at:&_target];
    [coder encodeValueOfObjCType:@encode(SEL) at:&_selector];
}

@end
