/*
 * TCSocket.m
 * Abstract superclass for socket objects.
 *
 * Copyright (c) 1997 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1997.
 */

/* NeXT headers */
#import <Foundation/Foundation.h>
#import <libc.h>

/* Project headers */
#import "TCNetworking.h"

@implementation TCSocket

- (id)initWithLocalAddress:(NSHost *)localAddress localPort:(unsigned)localPort remoteAddress:(NSHost *)remoteAddress remotePort:(unsigned)remotePort descriptor:(int)descriptor
{
    if (self = [super initWithFileDescriptor:descriptor]) {
        _localAddress = [localAddress retain];
        _localPort = localPort;
        _remoteAddress = [remoteAddress retain];
        _remotePort = remotePort;
    }
    
    return self;
}

- (void)dealloc
{
    [_localAddress release];
    [_remoteAddress release];

    [super dealloc];
}

- (NSString *)description
{
    return [NSString stringWithFormat:@"TCSocket local=(%d@%@) remote=(%d@%@) descriptor=%d",
        _localPort, [_localAddress description], _remotePort, [_remoteAddress description],
        [self fileDescriptor]];
}

- (id)initWithCoder:(NSCoder *)decoder
{
    if (self = [super initWithCoder:decoder]) {
        _localAddress = [[decoder decodeObject] retain];
        [decoder decodeValueOfObjCType:@encode(unsigned) at:&_localPort];
        _remoteAddress = [[decoder decodeObject] retain];
        [decoder decodeValueOfObjCType:@encode(unsigned) at:&_remotePort];
    }

    return self;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    [super encodeWithCoder:coder];

    [coder encodeObject:_localAddress];
    [coder encodeValueOfObjCType:@encode(unsigned) at:&_localPort];
    [coder encodeObject:_remoteAddress];
    [coder encodeValueOfObjCType:@encode(unsigned) at:&_remotePort];
}

- (NSHost *)remoteAddress
{
    return _remoteAddress;
}

- (unsigned)remotePort
{
    return _remotePort;
}

- (NSHost *)localAddress
{
    return _localAddress;
}

- (unsigned)localPort
{
    return _localPort;
}

@end
