/*
 * TCSocketConnector.h
 *
 *
 * Copyright (c) 1997 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1997.
 */

@class NSHost, TCSocket;

/* Exceptions */
TCNETWORKING_EXPORT NSString *TCSocketConnectorInvalidDestinationException;

/* Notifications */
TCNETWORKING_EXPORT NSString *TCSocketConnectorCompletionNotification;

/* UserInfo Keys */
TCNETWORKING_EXPORT NSString *TCSocketConnectorUserInfoExceptionNameItemKey;
TCNETWORKING_EXPORT NSString *TCSocketConnectorUserInfoExceptionReasonItemKey;

@interface TCSocketConnector : NSObject
{
    @private
    NSPort *_port;
    NSMutableDictionary *_operationUserInfo;
}

+ defaultConnector;

/* Returns a new descriptor; subclasses may use to provide alternate socket types; SOCK_STREAM implemented here */
- (int)newDescriptor;

/* Returns a newly connected socket; will raise an TCSocketConnectorGenericException on error */
- (TCSocket *)socketConnectedToHost:(NSHost *)address port:(unsigned)port;

/* Cover method; performs synchronous lookup on name and calls socketConnectedToHost:port: */
- (TCSocket *)socketConnectedToHostName:(NSString *)name port:(unsigned)port;

/* Asynchronous methods */
- (void)detachNewConnectorAddress:(NSHost *)address port:(unsigned)port object:(id)anObject selector:(SEL)aSelector;

/* Returns the userInfo from the most recent asynchronous operation */
- (NSDictionary *)operationUserInfo;

@end
