/*
 * TCStream.h
 * Generic stream. If NeXT doesn't have vision, well dammit, we do.
 *
 * Copyright (c) 1997 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1997.
 */

@interface TCStream : NSObject <NSCoding>
{
    @private
    int	_descriptor;

    struct _streamFlags {
        BOOL debug:1;
        BOOL closeOnDealloc:1;
    } _streamFlags;

    // added for TCTypedStream. default is the host byte order
    unsigned _endianness;
}

/* Conveniences */
+ (TCStream *)streamWithFileDescriptor:(int)descriptor;

/* Designated initializer */
- (id)initWithFileDescriptor:(int)descriptor;

/* Accessing flags */
- (BOOL)debug;
- (void)setDebug:(BOOL)aFlag;
- (BOOL)closeOnDealloc;
- (void)setCloseOnDealloc:(BOOL)aFlag;

/* Returns the number of bytes waiting in this descriptor's read buffer; reads of this length are guaranteed not to block */
- (unsigned)availableBytes;

/* Returns the raw descriptor */
- (int)fileDescriptor;

/* Closes the stream */
- (void)closeStream;

/* Attempts to read length bytes from the descriptor; this is not always possible, in the event of an error, truncates the data at the amount read. */
- (NSData *)readDataOfLength:(unsigned)length;
- (void)readDataOfLength:(unsigned)length at:(void *)ptr;

/* Reads whatever is available */
- (NSData *)readData;

/* Attempts to write data to the descriptor. Returns the number of bytes actually written. */
- (unsigned)writeData:(NSData *)data;
- (void)writeDataOfLength:(unsigned)length at:(void *)ptr;

@end

@interface TCStream(TypedStream)

- (void)setEndianness:(unsigned)order;
- (unsigned)endianness;

- (void)writeString:(NSString *)string;
- (void)writeFormat:(NSString *)aFormat, ...;
- (NSString *)readString;
- (NSString *)readLine;

- (int)readInt;
- (void)writeInt:(int)anInt;

- (unsigned int)readUnsignedInt;
- (void)writeUnsignedInt:(unsigned int)anInt;

- (short)readShort;
- (void)writeShort:(short)aShort;

- (unsigned short)readUnsignedShort;
- (void)writeUnsignedShort:(unsigned short)aShort;

- (unsigned char)readByte;
- (void)writeByte:(unsigned char)aByte;

- (void)readValueOfObjCType:(const char *)type at:(void *)ptr;
- (void)writeValueOfObjCType:(const char *)type at:(const void *)ptr;

@end

