#import "TCNetworking.h"

DEFINE_NSSTRING(TCStreamMonitorDataReadNotification);
DEFINE_NSSTRING(TCStreamMonitorRemoteClosedNotification);
DEFINE_NSSTRING(TCStreamMonitorNotificationDataItem);

@implementation TCStreamMonitor

- (id)initWithStream:(TCStream *)stream;
{
    if (self = [super init]) {
        _stream = [stream retain];
   }

    return self;
}

- (void)setDelegate:(id)delegate
{
    [_delegate release];
    _delegate = [delegate retain];
}

- (id)delegate
{
    return _delegate;
}

- (TCStream *)stream
{
    return _stream;
}

- (void)monitorInBackgroundAndNotify
{
    NSMutableArray *arguments = [NSMutableArray array];
    
    _dataPort = [[NSPort alloc] init];
    [_dataPort setDelegate:self];
    [[NSRunLoop currentRunLoop] addPort:_dataPort forMode:NSDefaultRunLoopMode];

    [arguments addObject:_stream];
    [arguments addObject:_dataPort];
    
    [NSThread detachNewThreadSelector:@selector(monitorWithPortsInArray:)
                             toTarget:[self class]
                           withObject:arguments];
}

- (void)dealloc
{
    [_stream release];
    [[NSRunLoop currentRunLoop] removePort:_dataPort forMode:NSDefaultRunLoopMode];
    [_dataPort release];
   
    [super dealloc];
}

@end

@implementation TCStreamMonitor(Private)

+ (void)monitorWithPortsInArray:(NSArray *)arguments
{
    NSAutoreleasePool *threadPool = [[NSAutoreleasePool alloc] init];
    TCSocket *socket = [[arguments objectAtIndex:0] retain];
    NSPort *dataPort = [[arguments objectAtIndex:1] retain];
    NSData *data = nil;

    NS_DURING {
        do {
            data = [socket readData];
            if (![dataPort sendData:data])
                [NSException raise:@"NSGenericException"
                            format:@"-[TCStreamMonitor monitorWithPortsInArray]: couldn't send mach message for %@", [data description]];
        }
        while ([data length] != 0);
    }
    NS_HANDLER {
        NSLog(@"exception during monitorWithPortsInArray: %@", [localException description]);
    }
    NS_ENDHANDLER;

    [socket release];
    [dataPort release];

    [threadPool release];
    [NSThread exit];
}

- (void)handlePortMessage:(NSPortMessage *)message
{
    NSData *data = [[message components] objectAtIndex:0];

    if ([_delegate respondsToSelector:@selector(handleSocketData:)])
        [_delegate handleSocketData:data];
    else {
        if ([data length] == 0) {
            [[NSNotificationCenter defaultCenter]
                postNotificationName:TCStreamMonitorRemoteClosedNotification
                              object:self];
        }
        else {
            [[NSNotificationCenter defaultCenter]
                postNotificationName:TCStreamMonitorDataReadNotification
                              object:self
                            userInfo:[NSDictionary dictionaryWithObjectsAndKeys:
                                data, TCStreamMonitorNotificationDataItem,
                                nil]];
        }
    }
}

@end
